@@ -33,9 +33,22 @@
 //!
 //! # Create an [`ObjectStore`] implementation:
 //!
-//! * [Google Cloud Storage](https://cloud.google.com/storage/): [`GoogleCloudStorageBuilder`](gcp::GoogleCloudStorageBuilder)
-//! * [Amazon S3](https://aws.amazon.com/s3/): [`AmazonS3Builder`](aws::AmazonS3Builder)
-//! * [Azure Blob Storage](https://azure.microsoft.com/en-gb/services/storage/blobs/):: [`MicrosoftAzureBuilder`](azure::MicrosoftAzureBuilder)
+#![cfg_attr(
+    feature = "gcp",
+    doc = "* [Google Cloud Storage](https://cloud.google.com/storage/): [`GoogleCloudStorageBuilder`](gcp::GoogleCloudStorageBuilder)"
+)]
+#![cfg_attr(
+    feature = "aws",
+    doc = "* [Amazon S3](https://aws.amazon.com/s3/): [`AmazonS3Builder`](aws::AmazonS3Builder)"
+)]
+#![cfg_attr(
+    feature = "azure",
+    doc = "* [Azure Blob Storage](https://azure.microsoft.com/en-gb/services/storage/blobs/): [`MicrosoftAzureBuilder`](azure::MicrosoftAzureBuilder)"
+)]
+#![cfg_attr(
+    feature = "http",
+    doc = "* [HTTP Storage](https://datatracker.ietf.org/doc/html/rfc2518): [`HttpBuilder`](http::HttpBuilder)"
+)]
 //! * In Memory: [`InMemory`](memory::InMemory)
 //! * Local filesystem: [`LocalFileSystem`](local::LocalFileSystem)
 //!
@@ -48,7 +61,7 @@
 //! * Concurrent Request Limit: [`LimitStore`](limit::LimitStore)
 //!
 //!
-//! # Listing objects:
+//! # List objects:
 //!
 //! Use the [`ObjectStore::list`] method to iterate over objects in
 //! remote storage or files in the local filesystem:
@@ -101,7 +114,7 @@
 //! ...
 //! ```
 //!
-//! # Fetching objects
+//! # Fetch objects
 //!
 //! Use the [`ObjectStore::get`] method to fetch the data bytes
 //! from remote storage or files in the local filesystem as a stream.
@@ -151,7 +164,57 @@
 //! ```text
 //! Num zeros in data/file01.parquet is 657
 //! ```
+//! #  Put object
+//! Use the [`ObjectStore::put`] method to save data in remote storage or local filesystem.
 //!
+//! ```
+//! # use object_store::local::LocalFileSystem;
+//! # fn get_object_store() -> LocalFileSystem {
+//! #     LocalFileSystem::new_with_prefix("/tmp").unwrap()
+//! # }
+//! # async fn put() {
+//!  use object_store::ObjectStore;
+//!  use std::sync::Arc;
+//!  use bytes::Bytes;
+//!  use object_store::path::Path;
+//!
+//!  let object_store: Arc<dyn ObjectStore> = Arc::new(get_object_store());
+//!  let path: Path = "data/file1".try_into().unwrap();
+//!  let bytes = Bytes::from_static(b"hello");
+//!  object_store
+//!      .put(&path, bytes)
+//!      .await
+//!      .unwrap();
+//! # }
+//! ```
+//!
+//! #  Multipart put object
+//! Use the [`ObjectStore::put_multipart`] method to save large amount of data in chunks.
+//!
+//! ```
+//! # use object_store::local::LocalFileSystem;
+//! # fn get_object_store() -> LocalFileSystem {
+//! #     LocalFileSystem::new_with_prefix("/tmp").unwrap()
+//! # }
+//! # async fn multi_upload() {
+//!  use object_store::ObjectStore;
+//!  use std::sync::Arc;
+//!  use bytes::Bytes;
+//!  use tokio::io::AsyncWriteExt;
+//!  use object_store::path::Path;
+//!
+//!  let object_store: Arc<dyn ObjectStore> = Arc::new(get_object_store());
+//!  let path: Path = "data/large_file".try_into().unwrap();
+//!  let (_id, mut writer) =  object_store
+//!      .put_multipart(&path)
+//!      .await
+//!      .unwrap();
+//!  let bytes = Bytes::from_static(b"hello");
+//!  writer.write_all(&bytes).await.unwrap();
+//!  writer.flush().await.unwrap();
+//!  writer.shutdown().await.unwrap();
+//! # }
+//! ```
 
 #[cfg(all(
     target_arch = "wasm32",
@@ -163,19 +226,25 @@ compile_error!("Features 'gcp', 'aws', 'azure' are not supported on wasm.");
 pub mod aws;
 #[cfg(feature = "azure")]
 pub mod azure;
+#[cfg(not(target_arch = "wasm32"))]
+pub mod chunked;
+pub mod delimited;
 #[cfg(feature = "gcp")]
 pub mod gcp;
+#[cfg(feature = "http")]
+pub mod http;
 pub mod limit;
 #[cfg(not(target_arch = "wasm32"))]
 pub mod local;
 pub mod memory;
 pub mod path;
+pub mod prefix;
 pub mod throttle;
 
-#[cfg(any(feature = "gcp", feature = "aws", feature = "azure"))]
+#[cfg(any(feature = "gcp", feature = "aws", feature = "azure", feature = "http"))]
 mod client;
 
-#[cfg(any(feature = "gcp", feature = "aws", feature = "azure"))]
+#[cfg(any(feature = "gcp", feature = "aws", feature = "azure", feature = "http"))]
 pub use client::{backoff::BackoffConfig, retry::RetryConfig};
 
 #[cfg(any(feature = "azure", feature = "aws", feature = "gcp"))]
@@ -197,7 +266,7 @@ use std::io::{Read, Seek, SeekFrom};
 use std::ops::Range;
 use tokio::io::AsyncWrite;
 
-#[cfg(any(feature = "azure", feature = "aws", feature = "gcp"))]
+#[cfg(any(feature = "azure", feature = "aws", feature = "gcp", feature = "http"))]
 pub use client::ClientOptions;
 
 /// An alias for a dynamically dispatched object store implementation.
@@ -486,6 +555,13 @@ pub enum Error {
 
     #[snafu(display("Operation not yet implemented."))]
     NotImplemented,
+
+    #[snafu(display(
+        "Configuration key: '{}' is not valid for store '{}'.",
+        key,
+        store
+    ))]
+    UnknownConfigurationKey { store: &'static str, key: String },
 }
 
 impl From<Error> for std::io::Error {
@@ -990,7 +1066,7 @@ mod tests {
         let paths = flatten_list_stream(storage, None).await.unwrap();
 
         for f in &paths {
-            let _ = storage.delete(f).await;
+            storage.delete(f).await.unwrap();
         }
     }
 
